#include "is_symmetric.h"

template <typename T>
IGL_INLINE bool igl::is_symmetric(const Eigen::SparseMatrix<T>& A)
{
  if(A.rows() != A.cols())
  {
    return false;
  }
  Eigen::SparseMatrix<T> AT = A.transpose();
  Eigen::SparseMatrix<T> AmAT = A-AT;
  //// Eigen screws up something with LLT if you try to do
  //SparseMatrix<T> AmAT = A-A.transpose();
  //// Eigen crashes at runtime if you try to do
  // return (A-A.transpose()).nonZeros() == 0;
  return AmAT.nonZeros() == 0;
}

template <typename DerivedA>
IGL_INLINE bool igl::is_symmetric(
  const Eigen::PlainObjectBase<DerivedA>& A)
{
  if(A.rows() != A.cols())
  {
    return false;
  }
  const typename Eigen::PlainObjectBase<DerivedA>& AT = A.transpose();
  const typename Eigen::PlainObjectBase<DerivedA>& AmAT = A-AT;
  //// Eigen screws up something with LLT if you try to do
  //SparseMatrix<T> AmAT = A-A.transpose();
  //// Eigen crashes at runtime if you try to do
  // return (A-A.transpose()).nonZeros() == 0;
  return AmAT.nonZeros() == 0;
}

#ifndef IGL_HEADER_ONLY
// Explicit template specialization
// generated by autoexplicit.sh
template bool igl::is_symmetric<Eigen::Matrix<double, -1, -1, 0, -1, -1> >(Eigen::PlainObjectBase<Eigen::Matrix<double, -1, -1, 0, -1, -1> > const&);
// generated by autoexplicit.sh
template bool igl::is_symmetric<double>(Eigen::SparseMatrix<double, 0, int> const&);
#endif
